# -*- coding: utf -*-

from __future__ import unicode_literals

import datetime
import time

from driverng.protocol.mllp import MLLPProtocol
from driverng.transport.base import TransportGiveUp, TransportTimedOutException
from driverng.utils import FS, byte2log


class MLLPProtocolHL7V2(MLLPProtocol):
    HL7_ACK = b"""MSH|^~\\&|%(sender_id)s|\"\"|%(receiver_id)s|\"\"|%(date)s||%(message_type)s|%(control_id)s|P|2.5.1|||NE|AL||UNICODE UTF-8|||%(profile_message)s\rMSA|AA|%(ack_id)s|\r"""
    HL7_QACK = b"""MSH|^~\\&|%(sender_id)s|\"\"|%(receiver_id)s|\"\"|%(date)s||%(message_type)s|%(control_id)s|P|2.5.1|||NE|||UNICODE UTF-8|||%(profile_message)s\rMSA|AA|%(ack_id)s\rQAK|%(qpd_id)s|OK|WOS^Work Order Step^IHELAW\rQPD|WOS^Work Order Step^IHELAW|%(qpd_id)s|%(qpd_sid)s"""

    def __init__(self, **kwargs):
        super(MLLPProtocolHL7V2, self).__init__(**kwargs)

    def mllp_readline_ack(self, timeout):
        return self.readline_ack(break_func=lambda d: d and d[-2:] == b"%c\r" % (FS,))

    def readline_ack(self, timeout=None, **kwargs):
        if timeout is not None:
            self.transport.set_timeout_port_sender(timeout)
        return self.transport.readline_ack(**kwargs)

    def recv_ack(self):
        # TODO(check ack)
        ack = self.mllp_readline_ack(timeout=1)
        self.logger.info("ack recu :%s", byte2log(ack))

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted_payload = payload.splitlines()
        splitted = payload.splitlines()[0].split(b"|")
        qpd_id = b""
        qpd_sid = b""
        for line in splitted_payload:
            splitted_qpd = b""
            if b"QPD" in line:
                splitted_qpd = line.split(b"|")
                qpd_id = splitted_qpd[2]
                qpd_sid = splitted_qpd[3]
        date = date or datetime.datetime.now()
        type_message = splitted[8]
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"date": date.strftime("%Y%m%d%H%M%S").encode(),
            b"control_id": control_id,
            b"sender_id": splitted[4],
            b"receiver_id": splitted[2],
            b"ack_id": splitted[9],
            b"profile_message": splitted[20][:10] if len(splitted) > 20 else "",
            b"qpd_id": qpd_id,
            b"qpd_sid": qpd_sid,
        }
        if self.is_automate:
            context[b"message_type"] = b"OML^O33^OML_O33"
        else:
            if type_message.startswith(b"QBP"):
                context[b"message_type"] = b"RSP^K11^RSP_K11"
                return MLLPProtocolHL7V2.HL7_QACK % context
            elif type_message.endswith(b"OUL_R24"):
                context[b"message_type"] = b"ACK^R24^ACK"
                return MLLPProtocolHL7V2.HL7_ACK % context
            elif type_message.endswith(b"SSU_U03"):
                context[b"message_type"] = b"ACK^U03^ACK"
                return MLLPProtocolHL7V2.HL7_ACK % context
            elif type_message.endswith(b"NMD_N02"):
                context[b"message_type"] = b"ACK^N02^ACK"
                return MLLPProtocolHL7V2.HL7_ACK % context
            if type_message.endswith(b"O33"):
                context[b"message_type"] = b"ACK^O33^ACK"
                return MLLPProtocolHL7V2.HL7_ACK % context
            else:
                context[b"message_type"] = b"ACK^R22^ACK"
                return MLLPProtocolHL7V2.HL7_ACK % context

    def recv_trames(self, timeout=None):
        data = self.mllp_readline(timeout)
        messages = []
        acks = []
        for d in data.split(b"\x1c\x0d"):  # FS CR
            if d:
                payload = self.get_payload(d)
                self.logger.info("recv trames: %s ", payload)
                ack = self.get_ack_message(payload)
                if ack:
                    acks.append(ack)
                messages.append(payload)
        for ack in acks:
            self.write(self.build_trames(ack))
        return messages

    def send_trames(self, trames):
        self.write(trames)
        for essai in range(10):
            try:
                self.transport.set_timeout_port_sender(1)
                self.recv_ack()
                return True

            except TransportTimedOutException:
                if self.transport.transport_recv is not None:
                    self.transport.set_timeout(1)
                    try:
                        self.logger.info(
                            "Non réception de l'acquittement sur le port d'émission, passage en lecture sur le port de réception"
                        )
                        messages = self.recv_trames()
                        if messages:
                            self.put_trames_to_mca(messages)
                    except TransportTimedOutException:
                        pass
                    except Exception as e:
                        self.logger.debug("Erreur sur le port de réception: %s", e)
                        raise

        raise TransportGiveUp("Après 10 tentatives, pas d'acquittement lu, abandon")

    @staticmethod
    def get_payload(data):
        return data[1:]
